# Copyright (c) 2001, Gemini Data Loggers (UK) Ltd

import time, string
import asyncore, asynchat

class Messaging(asynchat.async_chat):
    def __init__(*args):
        self = args[0]
        apply(asynchat.async_chat.__init__,args)
        self.set_terminator('\n')
        self._collected_data = ''
        self._handling_message = 0

    # asynchat handlers

    def collect_incoming_data(self,i):
        self._collected_data += i

    def found_terminator(self):
        data,self._collected_data = self._collected_data,''
        if self.terminator == '\n':
            while data and data[-1:] in '\r\n ':
                data = data[:-1]
            try:
                pos = string.index(data,' ')
            except ValueError:
                cmd,params = data, ''
            else:
                cmd,params = data[:pos], data[pos+1:]
            if cmd:
                self.handle_message(cmd,params)
        else:
            self.set_terminator('\n')
            self.handle_message_with_data(self.last_cmd,self.last_params,data)

    # message dispatch

    def handle_message(self,cmd,params):
        try:
            fn = getattr(self,'handle_message_'+string.lower(cmd))  
        except AttributeError:
            self.log_info('ERROR: unhandled message %r' % (cmd,))
            self.close()
        else:
            self._handling_message = 1
            try:
                fn(cmd,params)
            finally:
                self._handling_message = 0

    def handle_message_with_data(self,cmd,params,data):
        try:
            fn = getattr(self,'handle_message_'+string.lower(cmd))  
        except AttributeError:
            self.log_info('unhandled message %r' % (cmd,))
        else:
            fn(cmd,params,data)

    # API for derived classes

    def read_data(self,cmd,params,size):
        """Call this inside a message handler, to collect a fixed-length
        block-o-bytes from the input stream following the message
        """
        assert self._handling_message, 'can only collect data from inside a message handler'
        self.last_cmd = cmd
        self.last_params = params
        self.set_terminator(size)

    # standard messages

    def handle_message_log(self,cmd,params):
        self.log_info('logged: %r' % (params,))

    def handle_message_echo(self,cmd,params):
        self.push(params+'\n')

    def handle_message_ignore(self,cmd,params):
        pass
